#ifndef INCLUDE_MENUITEM_DEF
#define INCLUDE_MENUITEM_DEF
#endif
#include <qwidget.h>
#include <qmenudata.h>
#include <qpalette.h>
#include <qbitmap.h>
#include <qtabbar.h>
#include <qpointarray.h>

#include <ktoolbar.h>
#include <kapp.h>
#include <kdrawutil.h>
#include <klocale.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <kglobalsettings.h>
#include "thinKDE.h"

static QWidget *_lastWidget = 0;
static const int _indicatorSize = 14;
static const int _scrollBarExtent = 14;

extern "C"
{
	KStyle *allocate ()
	{
		return new thinKDE;
	}
	int minor_version ()
	{
		return 0;
	}
	int major_version ()
	{
		return 1;
	}
	const char *description ()
	{
		return (i18n ("SkelDesc").utf8 ());
	}
}

thinKDE::thinKDE ():KStyle ()
{
}

thinKDE::~thinKDE ()
{
	// Empty.
}

void thinKDE::polish (QApplication * app)
{
	KStyle::polish (app);
}

void thinKDE::polish (QPalette & pal)
{
	KStyle::polish (pal);
}

void thinKDE::unPolish (QApplication * app)
{
	KStyle::unPolish (app);
}

void thinKDE::polish (QWidget * w)
{
	//QWindowsStyle::polish(w);
	if (w->inherits ("QPushButton") || w->inherits ("QSlider")
		|| w->inherits ("QScrollBar"))
		w->installEventFilter (this);
}

void thinKDE::unPolish (QWidget * w)
{
	KStyle::unPolish (w);
}

void thinKDE::drawButton
	(QPainter * p,
	int x,
	int y,
	int w, int h, const QColorGroup & g, bool sunken, const QBrush * fill)
{
	int x2 = x + w - 1;
	int y2 = y + h - 1;
	if (!sunken)
	{

		p->fillRect (x, y, w, h, *fill);

		p->setPen (g.dark ().light (110));
		p->drawLine (x2, y, x2, y2);
		p->drawLine (x, y2, x2, y2);
		p->setPen (g.light ().light (120));
		p->drawLine (x, y, x2, y);
		p->drawLine (x, y, x, y2);
	}
	else
	{
		p->fillRect (x, y, w, h, g.button ().dark (108));
		p->setPen (g.dark ().dark (130));
		p->drawRect (x, y, w, h);
		p->setPen (g.dark ().dark (120));
		p->drawLine (x, y, x2, y);
		p->drawLine (x, y, x, y2);
		p->setPen (g.light ().light (120));
		p->drawLine (x2 - 1, y + 1, x2 - 1, y2 - 1);
		p->drawLine (x + 1, y2 - 1, x2 - 1, y2 - 1);
	}
}

QRect thinKDE::buttonRect (int x, int y, int w, int h)
{
	return KStyle::buttonRect (x, y, w, h);
}

void thinKDE::drawBevelButton
	(QPainter * p,
	int x,
	int y,
	int w, int h, const QColorGroup & g, bool sunken, const QBrush * fill)
{
	KStyle::drawBevelButton (p, x, y, w, h, g, sunken, fill);
}

void thinKDE::drawPushButton (QPushButton * b, QPainter * p)
{
	bool hl (_lastWidget == b);

	if (!hl)
	{
		KStyle::drawPushButton (b, p);
	}
	else
	{
		p->save ();
		QBrush *l = new QBrush (b->colorGroup ().button ().light (107));	//new QBrush(QColor("#E5E5F7"));
		drawButton (p, 0, 0, b->width (), b->height (), b->colorGroup (),
			b->isDown (), l);
		delete l;
		p->restore ();
	}

}

void thinKDE::drawPushButtonLabel (QPushButton * b, QPainter * p)
{
	KStyle::drawPushButtonLabel (b, p);
}

static void
	scrollBarControlsMetrics
	(const QScrollBar * sb, int sliderStart, int /* sliderMin */ ,
	int sliderMax,
	int sliderLength,
	int buttonDim,
	QRect & rSub,
	QRect & rAdd, QRect & rSubPage, QRect & rAddPage, QRect & rSlider)
{
	bool horizontal = sb->orientation () == QScrollBar::Horizontal;

	int len = horizontal ? sb->width () : sb->height ();

	int extent = horizontal ? sb->height () : sb->width ();

	QColorGroup g = sb->colorGroup ();

	if (sliderStart > sliderMax)
		sliderStart = sliderMax;

	int sliderEnd = sliderStart + sliderLength;

	int addX, addY;
	int subX, subY;
	int subPageX, subPageY, subPageW, subPageH;
	int addPageX, addPageY, addPageW, addPageH;
	int sliderX, sliderY, sliderW, sliderH;

	if (horizontal)
	{
		subY = 0;
		addY = 0;
		subX = 0;
		addX = buttonDim;

		subPageX = buttonDim * 2;
		subPageY = 0;
		subPageW = sliderStart - 1;
		subPageH = extent;

		addPageX = sliderEnd;
		addPageY = 0;
		addPageW = len - sliderEnd;
		addPageH = extent;

		sliderX = sliderStart;
		sliderY = 0;
		sliderW = sliderLength;
		sliderH = extent;
	}
	else
	{
		subX = 0;
		addX = 0;
		subY = len - buttonDim * 2;
		addY = len - buttonDim;

		subPageX = 0;
		subPageY = 0;
		subPageW = extent;
		subPageH = sliderStart;

		addPageX = 0;
		addPageY = sliderEnd;
		addPageW = extent;
		addPageH = subY - sliderEnd;

		sliderX = 0;
		sliderY = sliderStart;
		sliderW = extent;
		sliderH = sliderLength;
	}

	rSub.setRect (subX, subY, buttonDim, buttonDim);
	rAdd.setRect (addX, addY, buttonDim, buttonDim);
	rSubPage.setRect (subPageX, subPageY, subPageW, subPageH);
	rAddPage.setRect (addPageX, addPageY, addPageW, addPageH);
	rSlider.setRect (sliderX, sliderY, sliderW, sliderH);
}

static void drawScrollButton (QPainter * p, thinKDE * k, uint type,
	uint controls, uint activeControl, QRect r, Qt::ArrowType t, QColorGroup g,
	bool hl)
{
	if (controls & type && r.isValid ())
	{
		bool active (activeControl & type);
		p->fillRect (r, active ? g.background ().dark (109) : g.background ());
		p->setPen (g.dark ());
		p->drawLine (r.left (), r.top (), r.left (), r.bottom ());

		p->setPen (active ? g.dark () : g.light ().light (120));

		p->drawLine (r.left () + 2, r.top () + 2, r.right () - 1, r.top () + 2);
		p->drawLine (r.left () + 2, r.top () + 2, r.left () + 2,
			r.bottom () - 1);

		p->setPen (active ? g.light ().light (120) : g.dark ().light (110));
		p->drawLine (r.right () - 1, r.top () + 2, r.right () - 1,
			r.bottom () - 1);

		p->drawLine (r.left () + 2, r.bottom () - 1, r.right () - 2,
			r.bottom () - 1);

		k->drawArrow (p, t, active,	// FIXME - down ?
			r.x () + 2, r.y () + 3, r.width (), r.height (), g, true	// FIXME - enabled ?
			);
	}
}

void thinKDE::drawScrollBarControls
	(QPainter * p,
	const QScrollBar * sb, int sliderStart, uint controls, uint activeControl)
{
	//KStyle::drawScrollBarControls(p, sb, sliderStart, controls, activeControl);
	p->save ();
	bool hl (_lastWidget == (QWidget *)sb);
	//bool hl (_lastWidget == (QWidget *) p->device ());
	int sliderMin, sliderMax, sliderLength, buttonDim;

	scrollBarMetrics (sb, sliderMin, sliderMax, sliderLength, buttonDim);

	QRect rSub, rAdd, rSubPage, rAddPage, rSlider;

	scrollBarControlsMetrics
		(sb,
		sliderStart,
		sliderMin,
		sliderMax,
		sliderLength, buttonDim, rSub, rAdd, rSubPage, rAddPage, rSlider);

	QColorGroup g (sb->colorGroup ());
	Qt::ArrowType t =
		sb->orientation () == Horizontal ? Qt::RightArrow : Qt::DownArrow;

	drawScrollButton (p, this, (uint) AddLine, controls, activeControl, rAdd, t,
		g, hl);
	drawScrollButton (p, this, (uint) SubLine, controls, activeControl, rSub, t,
		g, hl);

	if (controls & SubPage && rSubPage.isValid ())
	{
		p->fillRect (rSubPage, QBrush (g.background ().dark (109)));
		p->setPen (g.dark ());
		p->drawLine (rSubPage.left (), rSubPage.top (), rSubPage.left (),
			rSubPage.bottom ());
	}

	if (controls & AddPage && rAddPage.isValid ())
	{
		p->fillRect (rAddPage, QBrush (g.background ().dark (109)));
		p->setPen (g.dark ());
		p->drawLine (rAddPage.left (), rAddPage.top (), rAddPage.left (),
			rAddPage.bottom ());
	}

	if (controls & Slider && rSlider.isValid ())
	{
		QBrush f ((activeControl & Slider)
			|| (hl) ? g.background ().light (107) : g.background ());
		if (Qt::Horizontal != sb->orientation ())
		{
			p->setPen (activeControl & Slider ? g.background ().light (107) : g.
				background ());
			for (int x = 4; x < 9; x++)
			{
				p->drawLine (rSlider.left () + x, rSlider.top () + (10 - x),
					rSlider.left () + 8, rSlider.top () + (10 - x));
			}
			//if (rSlider.top() - rSlider.bottom() > 14)
			for (int x = 7; x > 1; x--)
			{
				p->drawLine (rSlider.left () + 7, rSlider.bottom () - x,
					rSlider.right () - (10 - x), rSlider.bottom () - x);
			}
			//}
			p->fillRect (rSlider.x () + 9, rSlider.y () + 1, 5, 6, f);
			p->fillRect (rSlider.x () + 3, rSlider.bottom () - 7, 4, 7, f);
			p->fillRect (rSlider.x () + 3, rSlider.y () + 7,
				rSlider.width () - 5, rSlider.height () - 15, f);
			p->fillRect (rSlider, QBrush (g.background ().dark (109)));
			p->setPen (g.dark ().light (110));
			p->drawLine (rSlider.left (), rSlider.top (), rSlider.left (),
				rSlider.bottom ());
			p->drawLine (rSlider.right () - 1, rSlider.top (),
				rSlider.right () - 1, rSlider.bottom () - 8);

			p->drawLine (rSlider.left () + 2, rSlider.bottom (),
				rSlider.left () + 6, rSlider.bottom ());
			p->drawLine (rSlider.left () + 6, rSlider.bottom (), rSlider.right () - 2, rSlider.bottom () - 7);	//slash

			p->setPen (g.light ().light (130));

			p->drawLine (rSlider.right () - 2, rSlider.top (),
				rSlider.right () - 6, rSlider.top ());
			p->drawLine (rSlider.left () + 2, rSlider.top () + 7,
				rSlider.right () - 6, rSlider.top ());

			p->drawLine (rSlider.left () + 2, rSlider.top () + 7,
				rSlider.left () + 2, rSlider.bottom () - 1);

			QPoint center (rSlider.center ());
			if ((center.y () - rSlider.top ()) > 15)
			{
				p->setPen (g.light ().light (110));
				p->drawLine (rSlider.left () + 5, center.y () - 1 + 2,
					rSlider.right () - 4, center.y () - 7 + 2);
				p->drawLine (rSlider.left () + 5, center.y () + 3 + 2,
					rSlider.right () - 4, center.y () - 3 + 2);
				p->setPen (g.dark ());
				p->drawLine (rSlider.left () + 5, center.y () - 2 + 2,
					rSlider.right () - 4, center.y () - 8 + 2);
				p->drawLine (rSlider.left () + 5, center.y () + 2 + 2,
					rSlider.right () - 4, center.y () - 4 + 2);
			}
		}
	}

	p->restore ();

}

QStyle::ScrollControl
	thinKDE::scrollBarPointOver
	(const QScrollBar * sb, int sliderStart, const QPoint & point)
{
	return KStyle::scrollBarPointOver (sb, sliderStart, point);
}

void thinKDE::scrollBarMetrics
	(const QScrollBar * sb,
	int &sliderMin, int &sliderMax, int &sliderLength, int &buttonDim)
{
	int maxlen;

	bool horizontal = sb->orientation () == QScrollBar::Horizontal;

	int len = (horizontal) ? sb->width () : sb->height ();

	int extent = (horizontal) ? sb->height () : sb->width ();

	if (len > (extent - 1) * 2)
		buttonDim = extent;
	else
		buttonDim = len / 2 - 1;

	if (horizontal)
		sliderMin = buttonDim * 2;
	else
		sliderMin = 1;

	maxlen = len - buttonDim * 2 - 1;

	sliderLength =
		(sb->pageStep () * maxlen) /
		(sb->maxValue () - sb->minValue () + sb->pageStep ());

	if (sliderLength < _scrollBarExtent)
		sliderLength = _scrollBarExtent;

	if (sliderLength > maxlen)
		sliderLength = maxlen;

	sliderMax = sliderMin + maxlen - sliderLength;

}

QSize thinKDE::indicatorSize () const
{
	return KStyle::indicatorSize ();
}

void thinKDE::drawIndicator
	(QPainter * p,
	int x,
	int y,
	int w, int h, const QColorGroup & g, int state, bool down, bool enabled)
{
	KStyle::drawIndicator (p, x, y, w, h, g, state, down, enabled);
}

QSize thinKDE::exclusiveIndicatorSize () const
{
	return KStyle::exclusiveIndicatorSize ();
}

void thinKDE::drawExclusiveIndicator
	(QPainter * p,
	int x,
	int y,
	int w, int h, const QColorGroup & g, bool on, bool down, bool enabled)
{
	KStyle::drawExclusiveIndicator (p, x, y, w, h, g, on, down, enabled);
}

void thinKDE::drawIndicatorMask
	(QPainter * p, int x, int y, int w, int h, int state)
{
	KStyle::drawIndicatorMask (p, x, y, w, h, state);
}

void thinKDE::drawExclusiveIndicatorMask
	(QPainter * p, int x, int y, int w, int h, bool on)
{
	KStyle::drawExclusiveIndicatorMask (p, x, y, w, h, on);
}

void thinKDE::drawComboButton
	(QPainter * p,
	int x,
	int y,
	int w,
	int h,
	const QColorGroup & g,
	bool sunken, bool editable, bool enabled, const QBrush * fill)
{
	KStyle::drawComboButton (p, x, y, w, h, g, sunken, editable, enabled, fill);
}

QRect thinKDE::comboButtonRect (int x, int y, int w, int h)
{
	return KStyle::comboButtonRect (x, y, w, h);
}

QRect thinKDE::comboButtonFocusRect (int x, int y, int w, int h)
{
	return KStyle::comboButtonFocusRect (x, y, w, h);
}

int thinKDE::sliderLength () const
{
	return KStyle::sliderLength ();
}

void thinKDE::drawSliderGroove
	(QPainter * p,
	int x, int y, int w, int h, const QColorGroup & g, QCOORD c, Orientation o)
{
	p->fillRect (x, y, w, h, g.dark ().light (168));

	int x2 = x + w - 1;
	int y2 = y + h - 1;
	p->setPen (g.light ());
	p->drawLine (x2, y, x2, y2);
	p->drawLine (x, y2, x2, y2);
	p->setPen (g.dark ().light (110));
	p->drawLine (x, y, x2, y);
	p->drawLine (x, y, x, y2);
}

void thinKDE::drawArrow
	(QPainter * p,
	Qt::ArrowType type,
	bool down,
	int x,
	int y,
	int w, int h, const QColorGroup & g, bool enabled, const QBrush * fill)
{
	p->save ();
	if (!down)
	{
		p->setPen (g.light ().light (110));
		p->drawLine (x + 3, y + 9, x + 9, y + 3);
		p->setPen (g.dark ());
		p->drawLine (x + 3, y + 8, x + 8, y + 3);
	}
	else
	{
		p->setPen (g.dark ());
		p->drawLine (x + 3, y + 9, x + 9, y + 3);
	}
	p->restore ();
}

void thinKDE::drawSlider
	(QPainter * p,
	int x,
	int y,
	int w,
	int h, const QColorGroup & g, Orientation o, bool tickAbove, bool tickBelow)
{
	bool hl (_lastWidget == (QWidget *) p->device ());

	if (!hl)
	{
		p->fillRect (x, y + 2, w, h - 4, g.background ());
	}
	else
	{
		p->fillRect (x, y + 2, w, h - 4,
			QBrush (((QWidget *) p->device ())->colorGroup ().background ().
				light (107)));
	}

	int x2 = x + w - 1;
	int y2 = y + h - 1 - 2;
	p->setPen (g.dark ());
	p->drawLine (x2, y + 3, x2, y2);
	p->drawLine (x, y2, x2, y2);

	p->setPen (g.light ());
	p->drawLine (x, y + 2, x2, y + 2);
	p->drawLine (x, y + 2, x, y2 - 1);

	if (o == Horizontal)
	{
		p->setPen (g.dark ());
		p->drawLine (x + 8, y + 5, x + 8, y2 - 3);
		p->setPen (g.light ());
		p->drawLine (x + 9, y + 5, x + 9, y2 - 3);
	}
	else
	{
		p->setPen (g.dark ());
		p->drawLine (x + 4, y + 8 + 3, x2 - 4, y + 8);
		p->setPen (g.light ());
		p->drawLine (x + 4, y + 9 + 3, x2 - 4, y + 9);
	}
}

void thinKDE::drawKToolBar
	(QPainter * p,
	int x,
	int y, int w, int h, const QColorGroup & g, KToolBarPos pos, QBrush * fill)
{
	p->save ();
	p->fillRect (x, y, w, h, g.background ());
	p->setPen (g.dark ().light (120));
	p->drawLine (x, h - 1, w, h - 1);	//down shadow
	p->setPen (g.light ().light (120));
	p->drawLine (x, y, w, y);
	p->drawLine (x, y, x, h);
	p->restore ();
}

void thinKDE::drawKBarHandle
	(QPainter * p,
	int x,
	int y, int w, int h, const QColorGroup & g, KToolBarPos pos, QBrush * fill)
{
	int x2 = x + w;
	int y2 = y + h;
	p->setPen (g.light ().light (120));
	p->drawLine (x, y, x2, y);
	p->drawLine (x, y, x, y2);
	p->drawLine (x2 - 1, y, x2 - 1, y2 - 1);
	p->setPen (g.dark ().light (120));
	p->drawLine (x2 - 2, y, x2 - 2, y2 - 1);
	p->drawLine (x + 1, y2 - 1, x2 - 1, y2 - 1);
	if ((pos == Top) || (pos == Bottom))
	{
		p->setPen (g.dark ().light (120));
		p->drawLine (x + 2, (y + h) / 2 - 2, x + 5, (y + h) / 2 - 5);
		p->drawLine (x + 2, (y + h) / 2 + 2, x + 5, (y + h) / 2 - 1);
		p->setPen (g.light ().light (120));
		p->drawLine (x + 2, (y + h) / 2 - 1, x + 5, (y + h) / 2 - 4);
		p->drawLine (x + 2, (y + h) / 2 + 3, x + 5, (y + h) / 2);
	}
	else if ((pos == Right) || (pos == Left))
	{
		p->setPen (g.dark ().light (120));
		p->drawLine (x + 2, (y + h) / 2 - 2, x + 5, (y + h) / 2 - 5);
		p->drawLine (x + 2, (y + h) / 2 + 2, x + 5, (y + h) / 2 - 1);
		p->setPen (g.light ().light (120));
		p->drawLine (x + 2, (y + h) / 2 - 1, x + 5, (y + h) / 2 - 4);
		p->drawLine (x + 2, (y + h) / 2 + 3, x + 5, (y + h) / 2);
	}
}

void thinKDE::drawKMenuBar
	(QPainter * p,
	int x,
	int y, int w, int h, const QColorGroup & g, bool macMode, QBrush * fill)
{
	p->fillRect (x, y, w, h, g.brush (QColorGroup::Background));
	p->setPen (Qt::darkGray);
	p->drawLine (x, h - 1, w, h - 1);	//down shadow
	p->setPen (g.light ().light (120));
	p->drawLine (x, y, w, y);	//up highlight

}

void thinKDE::drawKToolBarButton
	(QPainter * p,
	int x,
	int y,
	int w,
	int h,
	const QColorGroup & g,
	bool sunken,
	bool raised,
	bool enabled,
	bool popup,
	KToolButtonType icontext,
	const QString & btext,
	const QPixmap * pixmap, QFont * font, QWidget * button)
{

	int dx, dy;
	int x2 = (x + w), y2 = (y + h);
	QFontMetrics fm (*font);

	if (raised || sunken)
	{
		if (raised)
		{
			p->setPen (g.background ().dark ());
			p->drawRect (x, y, w, h);
			p->fillRect (x + 1, y + 1, w - 2, h - 2,
				g.background ().light (107));
			p->setPen (g.dark ().light (120));
			p->drawLine (x2 - 1, y, x2 - 1, y2 - 1);
			p->drawLine (x + 1, y2 - 1, x2 - 1, y2 - 1);
			p->setPen (g.light ().light (120));
			p->drawLine (x, y, x2, y);
			p->drawLine (x, y, x, y2);
		}
		if (sunken)
		{
			p->setPen (g.background ().dark ());
			p->drawRect (x, y, w, h);
			p->fillRect (x + 1, y + 1, w - 2, h - 2,
				g.background ().dark (120));
			p->setPen (g.light ().light (107));
			p->drawLine (x2 - 1, y, x2 - 1, y2 - 1);
			p->drawLine (x + 1, y2 - 1, x2 - 1, y2 - 1);
			p->setPen (g.dark ().light (107));
			p->drawLine (x, y, x2, y);
			p->drawLine (x, y, x, y2);
		}
	}
	else
	{
		p->fillRect (x, y, w, h, g.brush (QColorGroup::Background));
	}

	p->setPen (g.buttonText ());

	if (icontext == Icon)
	{							// icon only
		if (pixmap)
		{
			dx = (w - pixmap->width ()) / 2 + 1;
			dy = (h - pixmap->height ()) / 2;
			if (sunken)
			{
				dx += 1;
				dy += 1;
				p->fillRect (x + 1, y + 1, w - 2, h - 2,
					g.background ().dark (120));
			}
			p->drawPixmap (x + dx, y + dy, *pixmap);
		}
	}
	else if (icontext == IconTextRight)
	{							// icon and text (if any)
		if (pixmap)
		{
			dx = 4;
			dy = (h - pixmap->height ()) / 2;
			if (sunken)
			{
				dx = 4;
				dy = (h - pixmap->height ()) / 2;
				p->fillRect (x + 1, y + 1, w - 2, h - 2,
					g.background ().dark (120));
			}
			p->drawPixmap (x + dx, y + dy, *pixmap);
		}

		if (!btext.isNull ())
		{
			int tf = AlignVCenter | AlignLeft;
			if (pixmap)
				dx = 4 + pixmap->width () + 2;
			else
				dx = 4;
			dy = 0;
			if (!enabled)
				p->setPen (g.dark ());
			if (font)
				p->setFont (*font);
			//p->setPen(g.text());
			if (raised || sunken)
				p->setPen (KGlobalSettings::toolBarHighlightColor ());
			p->drawText (x + dx, y + dy, w - dx, h, tf, btext);
		}
	}
	else if (icontext == Text)
	{							// only text, even if there is a icon
		if (!btext.isNull ())
		{
			int tf = AlignVCenter | AlignLeft;
			if (!enabled)
				p->setPen (g.dark ());
			dx = (w - fm.width (btext)) / 2;
			dy = (h - fm.lineSpacing ()) / 2;
			if (sunken)
			{
				++dx;
				++dy;
			}
			if (font)
				p->setFont (*font);
			//p->setPen(g.text());
			if (raised)
				p->setPen (KGlobalSettings::toolBarHighlightColor ());
			p->drawText (x + dx, y + dy, fm.width (btext),
				fm.lineSpacing (), tf, btext);
		}
	}
	else if (icontext == IconTextBottom)
	{
		if (pixmap)
		{
			dx = (w - pixmap->width ()) / 2;
			dy = (h - fm.lineSpacing () - pixmap->height ()) / 2;
			if (sunken)
			{
				++dx;
				++dy;
			}
			p->drawPixmap (x + dx, y + dy, *pixmap);
		}
		if (!btext.isNull ())
		{
			int tf = AlignBottom | AlignHCenter;
			dy = pixmap->height ();
			dx = 2;
			if (!enabled)
				p->setPen (g.dark ());
			if (sunken)
			{
				++dx;
				++dy;
			}
			if (font)
				p->setFont (*font);
			if (raised)
				p->setPen (KGlobalSettings::toolBarHighlightColor ());
			p->drawText (x, y, w, h - 3, tf, btext);
		}
	}
	if (popup)
	{
		if (enabled)
			qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle,
				false, w - 5, h - 5, 0, 0, g, true);
		else
			qDrawArrow (p, Qt::DownArrow, Qt::WindowsStyle,
				false, w - 5, h - 5, 0, 0, g, false);
	}
}

void thinKDE::drawKMenuItem
	(QPainter * p,
	int x,
	int y,
	int w,
	int h, const QColorGroup & g, bool active, QMenuItem * mi, QBrush * fill)
{

	if (active)
	{
		p->fillRect (x + 1, y + 1, w - 2, h - 2, g.background ().light (107));
		p->setPen (g.light ().light (120));
		p->drawLine (x, y, x + w, y);
		p->drawLine (x, y, x, y + h);
		p->setPen (g.dark ().light (120));
		p->drawLine (x + w - 1, y, x + w - 1, y + h - 1);
		p->drawLine (x + 1, y + h - 1, x + w - 1, y + h - 1);
	}

	QApplication::style ().drawItem (p,
		x,
		y,
		w,
		h,
		AlignCenter | ShowPrefix | DontClip | SingleLine,
		g, mi->isEnabled (), mi->pixmap (), mi->text (), -1, &g.text ());
}

void thinKDE::drawPopupMenuItem
	(QPainter * p,
	bool checkable,
	int maxpmw,
	int tab,
	QMenuItem * mi,
	const QPalette & pal, bool act, bool enabled, int x, int y, int w, int h)
{
	static const int motifItemFrame = 0;
	static const int motifItemHMargin = 7;
	static const int motifItemVMargin = 7;
	static const int motifArrowHMargin = 6;
	static const int windowsRightBorder = 12;

	bool dis = !enabled;
	QColorGroup itemg = dis ? pal.disabled () : pal.active ();

	maxpmw = QMAX (maxpmw, 20);
	int checkcol = maxpmw;

	if (!mi)
	{
		return;
	}

	if (mi->isSeparator ())
	{
		p->fillRect (x, y, w, h, itemg.background ());
		p->setPen (itemg.mid ());
		p->drawLine (x + 3, y + 0, x + w - 3, y + 0);
		p->setPen (itemg.light ().light (120));
		p->drawLine (x + 3, y + 1, x + w - 3, y + 1);
		//p->fillRect(x, y, 22, h, itemg.mid());
		return;
	}

	if (act && enabled)
	{
		// draw highlight around current menu item
		QPen oldPen = p->pen ();

		p->setPen (itemg.highlight ().dark ());
		p->setBrush (itemg.background ().light (108));
		p->drawRect (x, y, w, h);

		p->setPen (oldPen);
	}
	else
	{
		p->fillRect (x, y, w, h, itemg.background ());
		//p->fillRect (x, y, 22, h, itemg.background ());
	}
	if (checkable && !mi->iconSet () && mi->isChecked ())
	{

		p->save ();
		p->setPen (itemg.dark ());
		int w2 = checkcol - 2;
		int h2 = h - 2;

		const int ol = 10;
		const int ox = x + 2 + ((w2 - ol) / 2);
		const int oy = y + 1 + ((h2 - ol) / 2);

		if (w2 < 10 || h2 < 10)
			return;

		p->setBrush (itemg.highlight ());
		p->drawRect (ox, oy, ol, ol);
		p->restore ();
	}
	else if (mi->iconSet ())
	{
		// draw associated icon
		QIconSet::Mode mode = dis ? QIconSet::Disabled : QIconSet::Normal;
		if (!dis)
			mode = QIconSet::Active;
		QPixmap pixmap = mi->iconSet ()->pixmap (QIconSet::Small, mode);
		int pixw = pixmap.width ();
		int pixh = pixmap.height ();

		QRect cr (x, y, checkcol, h);
		QRect pmr (0, 0, pixw, pixh);
		pmr.moveCenter (cr.center ());
		p->setPen (itemg.highlightedText ());
		p->drawPixmap (pmr.topLeft (), pixmap);
	}

	p->setPen (itemg.buttonText ());
	QColor discol;
	if (dis)
	{
		discol = itemg.text ();
		p->setPen (discol);
	}

	int xm = motifItemFrame + checkcol + motifItemHMargin;

	QString s = mi->text ();
	if (!s.isNull ())
	{
		int t = s.find ('\t');
		int m = motifItemVMargin;
		const int text_flags =
			AlignVCenter | ShowPrefix | DontClip | SingleLine;
		if (t >= 0)
		{
			if (dis)
			{
				p->setPen (itemg.text ());
				p->drawText (x + w - tab -
					windowsRightBorder -
					motifItemHMargin -
					motifItemFrame + 1, y + m + 1,
					tab, h - 2 * m, text_flags, s.mid (t + 1));
				p->setPen (discol);
			}
			//if (act && enabled) {
			p->setPen (itemg.text ());
			//}/
			p->drawText (x + w - tab - windowsRightBorder -
				motifItemHMargin - motifItemFrame,
				y + m, tab, h - 2 * m, text_flags, s.mid (t + 1));
		}

		if (dis)
		{
			p->setPen (itemg.text ());
			p->drawText (x + xm + 1, y + m + 1, w - xm + 1,
				h - 2 * m, text_flags, s, t);
			p->setPen (discol);
		}
		if (act && enabled)
		{
			p->setPen (itemg.text ());
		}
		p->drawText (x + xm, y + m, w - xm - tab + 1, h - 2 * m,
			text_flags, s, t);
	}
	else if (mi->pixmap ())
	{
		QPixmap *pixmap = mi->pixmap ();
		if (pixmap->depth () == 1)
			p->setBackgroundMode (OpaqueMode);
		p->drawPixmap (x + xm, y + motifItemFrame, *pixmap);
		if (pixmap->depth () == 1)
			p->setBackgroundMode (TransparentMode);
	}

	if (mi->popup ())
	{
		int dim = (h - 2 * motifItemFrame) / 2;
		if (act)
		{
			if (!dis)
				discol = itemg.buttonText ();

			QColorGroup itemg2 (discol, itemg.highlight (),
				white, white, dis ? discol : white, discol, white);
			drawArrow (p, RightArrow, FALSE,
				x + w - motifArrowHMargin -
				motifItemFrame - dim,
				y + h / 2 - dim / 2, dim, dim, itemg2, TRUE);
		}
		else
		{
			drawArrow (p, RightArrow, FALSE,
				x + w - motifArrowHMargin -
				motifItemFrame - dim,
				y + h / 2 - dim / 2, dim, dim, itemg, mi->isEnabled ());

		}
	}
}

void thinKDE::drawKProgressBlock
	(QPainter * p,
	int x, int y, int w, int h, const QColorGroup & g, QBrush * fill)
{
	KStyle::drawKProgressBlock (p, x, y, w, h, g, fill);
}

void thinKDE::drawFocusRect
	(QPainter * p,
	const QRect & r, const QColorGroup & g, const QColor * pen, bool atBorder)
{
	KStyle::drawFocusRect (p, r, g, pen, atBorder);
}

void thinKDE::drawPanel
	(QPainter * p,
	int x,
	int y,
	int w,
	int h,
	const QColorGroup & g, bool sunken, int lineWidth, const QBrush * brush)
{
	KStyle::drawPanel (p, x, y, w, h, g, sunken, lineWidth, brush);
}

void thinKDE::drawPopupPanel
	(QPainter * p,
	int x,
	int y,
	int w, int h, const QColorGroup & g, int lineWidth, const QBrush * brush)
{
	KStyle::drawPopupPanel (p, x, y, w, h, g, lineWidth, brush);
}

void thinKDE::drawSeparator
	(QPainter * p,
	int x,
	int y,
	int w,
	int h, const QColorGroup & g, bool sunken, int lineWidth, int midLineWidth)
{
	KStyle::drawSeparator (p, x, y, w, h, g, sunken, lineWidth, midLineWidth);
}

void thinKDE::drawTab
	(QPainter * p, const QTabBar * tabBar, QTab * tab, bool selected)
{
	KStyle::drawTab (p, tabBar, tab, selected);
}

void thinKDE::drawTabMask
	(QPainter * p, const QTabBar * tabBar, QTab * tab, bool selected)
{
	KStyle::drawTabMask (p, tabBar, tab, selected);
}

void thinKDE::drawKickerHandle
	(QPainter * p,
	int x, int y, int w, int h, const QColorGroup & g, QBrush * brush)
{
	KStyle::drawKickerHandle (p, x, y, w, h, g, brush);
}

void thinKDE::drawKickerAppletHandle
	(QPainter * p,
	int x, int y, int w, int h, const QColorGroup & g, QBrush * brush)
{
	KStyle::drawKickerAppletHandle (p, x, y, w, h, g, brush);
}

void thinKDE::drawKickerTaskButton
	(QPainter * p,
	int x,
	int y,
	int w,
	int h,
	const QColorGroup & g,
	const QString & title, bool active, QPixmap * icon, QBrush * brush)
{
	KStyle::drawKickerTaskButton (p, x, y, w, h, g, title, active, icon, brush);
}

bool thinKDE::eventFilter (QObject * obj, QEvent * e)
{
	switch (e->type ())
	{
		case QEvent::Enter:
		{
			if (obj->inherits ("QPushButton") ||
				obj->inherits ("QComboBox") ||
				obj->inherits ("QSlider") || obj->inherits ("QScrollBar"))
			{
				_lastWidget = (QWidget *) obj;
				_lastWidget->repaint (FALSE);
			}
			break;
		}
		case QEvent::Leave:
		{
			if (obj == _lastWidget)
			{
				_lastWidget = 0;
				((QWidget *) obj)->repaint (FALSE);
				break;
			}
		}
		default:
		{
			break;
		}
	};

	return KStyle::eventFilter (obj, e);
}
